/*
  QtCurve (C) Craig Drummond, 2004 Craig.Drummond@lycos.co.uk


  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files(the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qtabwidget.h>
#include <qheader.h>
#include <kglobal.h>
#include <klocale.h>
#include <kcolorbutton.h>
#include <kpushbutton.h>
#include <kstdguiitem.h>
#include <kdialog.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <kinputdialog.h>
#include <unistd.h>
#include "qtcurveconfig.h"
#include "../../config.h"

extern "C"
{
    QWidget * allocate_kstyle_config(QWidget *parent)
    {
        KGlobal::locale()->insertCatalogue("kstyle_qtcurve_config");
        return new QtCurveConfig(parent);
    }
}

static const char *toStr(EDefBtnIndicator ind)
{
    switch(ind)
    {
        case IND_NONE:
            return "none";
        case IND_FONT_COLOR:
            return "fontcolor";
        default:
            return "corner";
    }
}

static const char *toStr(EHandle ind)
{
    switch(ind)
    {
        case HANDLE_RAISED:
            return "raised";
        case HANDLE_DOTS:
            return "dots";
        case HANDLE_DASHES:
            return "dashes";
        default:
            return "sunken";
    }
}

static const char *toStr(EGroove ind)
{
    switch(ind)
    {
        case GROOVE_RAISED:
            return "raised";
        case GROOVE_NONE:
            return "none";
        default:
            return "sunken";
    }
}

static const char *toStr(ETBarBorder ind)
{
    switch(ind)
    {
        case TB_DARK:
            return "dark";
        case TB_NONE:
            return "none";
        default:
            return "light";
    }
}

static const char *toStr(ELvExpander exp)
{
    switch(exp)
    {
        case LV_EXP_PM:
            return "plusminus";
        default:
            return "arrow";
    }
}

static const char *toStr(ELvLines exp)
{
    switch(exp)
    {
        case LV_LINES_NONE:
            return "none";
        case LV_LINES_DOTTED:
            return "dotted";
        default:
            return "solid";
    }
}

static const char *toStr(EProfile exp)
{
    switch(exp)
    {
        case PROFILE_FLAT:
            return "flat";
        case PROFILE_RAISED_GRADIENT:
            return "raised";
        case PROFILE_SUNKEN_GRADIENT:
            return "sunken";
        default:
            return "lightraised";
    }
}

static const char *toStr(EAppearance exp)
{
    switch(exp)
    {
        case APPEARANCE_FLAT:
            return "flat";
        case APPEARANCE_GRADIENT:
            return "gradient";
        case APPEARANCE_LIGHT_GRADIENT:
            return "lightgradient";
        case APPEARANCE_DULL_GLASS:
            return "dullglass";
        case APPEARANCE_BEVELLED:
            return "bevelled";
        case APPEARANCE_INVERTED:
            return "inverted";
        default:
            return "shinyglass";
    }
}

static const char *toStr(EShade exp)
{
    switch(exp)
    {
        default:
        case SHADE_NONE:
            return "none";
        case SHADE_SELECTED:
            return "selected";
        case SHADE_CUSTOM:
            return "custom";
        case SHADE_DARKEN:
            return "darken";
    }
}

static const char *toStr(ESeparator exp)
{
    switch(exp)
    {
        default:
        case SEPARATOR_LINES:
            return "lines";
        case SEPARATOR_NONE:
            return "none";
        case SEPARATOR_DOTS:
            return "dots";
    }
}

static const char *toStr(EHeaderColor exp)
{
    switch(exp)
    {
        default:
        case HEADER_COLOR_BGND:
            return "background";
        case HEADER_COLOR_BTN:
            return "button";
        case HEADER_COLOR_CUSTOM:
            return "custom";
    }
}

static const char *toStr(EFocus exp)
{
    switch(exp)
    {
        default:
        case FOCUS_DOTS:
            return "dots";
        case FOCUS_COLORED:
            return "colored";
    }
}

static void insertShadeEntries(QComboBox *combo, bool withDarken)
{
    combo->insertItem(i18n("None"));
    combo->insertItem(i18n("'Selected background'"));
    combo->insertItem(i18n("Custom:"));
    if(withDarken)
        combo->insertItem(i18n("Darken"));
}

static void insertAppearanceEntries(QComboBox *combo, bool all)
{
    combo->insertItem(i18n("Flat"));
    combo->insertItem(i18n("Dull glass"));
    combo->insertItem(i18n("Shiny glass"));
    combo->insertItem(i18n("Gradient"));
    combo->insertItem(i18n("Inverted gradient"));
    if(all)
    {
        combo->insertItem(i18n("Light gradient"));
        combo->insertItem(i18n("Bevelled"));
    }
}

static void insertProfileEntries(QComboBox *combo)
{
    combo->insertItem(i18n("Flat"));
    combo->insertItem(i18n("Raised gradient"));
    combo->insertItem(i18n("Sunken gradient"));
    combo->insertItem(i18n("Raised light gradient"));
}

static void insertHeaderColorEntries(QComboBox *combo)
{
    combo->insertItem(i18n("Background"));
    combo->insertItem(i18n("Button"));
    combo->insertItem(i18n("Custom:"));
}

static void insertHandleEntries(QComboBox *combo)
{
    combo->insertItem(i18n("Raised lines"));
    combo->insertItem(i18n("Sunken lines"));
    combo->insertItem(i18n("Dots"));
    combo->insertItem(i18n("Dashes"));
}

static void insertFocusEntries(QComboBox *combo)
{
    combo->insertItem(i18n("Dots"));
    combo->insertItem(i18n("Colored"));
}

QtCurveConfig::ListViewItem::ListViewItem(const Options &o, KListView *parent, bool t)
                           : KListViewItem(parent),
                             opts(o),
                             top(t)
{
    setText(0, opts.name);
}

QString QtCurveConfig::ListViewItem::key(int column, bool ascending) const
{
    return QString((top ? ascending ? "0" : "2" : "1") + text(column));
}

QtCurveConfig::QtCurveConfig(QWidget *parent)
             : QtCurveConfigBase(parent),
               currentStyle(0),
               defaultStyle(0)
{
    titleLabel->setText("QtCurve " VERSION " - (C) Craig Drummond, 2003-2006");
    insertShadeEntries(shadeSlidersCombo, false);
    insertShadeEntries(shadeMenubarsCombo, true);
    insertAppearanceEntries(appearanceCombo, true);
    insertAppearanceEntries(menubarAppearanceCombo, false);
    insertAppearanceEntries(toolbarAppearanceCombo, false);
    insertAppearanceEntries(lvHeaderAppearanceCombo, true);
    insertProfileEntries(menuitemProfileCombo);
    insertProfileEntries(progressbarProfileCombo);
    insertHeaderColorEntries(lvHeaderColorCombo);
    insertHandleEntries(handlesCombo);
    insertFocusEntries(focusCombo);

    versionWidget->header()->hide();

    KPushButton *save=new KPushButton(KGuiItem(i18n("Save"), "filesave", i18n("Save style")), controlsFrame),
                *imp=new KPushButton(KGuiItem(i18n("Import..."), "fileimport", i18n("Import style")), controlsFrame),
                *exp=new KPushButton(KGuiItem(i18n("Export..."), "fileexport", i18n("Export style")), controlsFrame);
    QGridLayout *controlsLayout = new QGridLayout(controlsFrame, 2, 2, 0, KDialog::spacingHint());

    delBtn=new KPushButton(KStdGuiItem::del(), controlsFrame),
    controlsLayout->addWidget(save, 0, 0);
    controlsLayout->addWidget(delBtn, 0, 1);
    controlsLayout->addWidget(imp, 1, 0);
    controlsLayout->addWidget(exp, 1, 1);

    connect(save, SIGNAL(clicked()), this, SLOT(saveStyle()));
    connect(delBtn, SIGNAL(clicked()), this, SLOT(deleteStyle()));
    connect(imp, SIGNAL(clicked()), this, SLOT(importStyle()));
    connect(exp, SIGNAL(clicked()), this, SLOT(exportStyle()));

    loadStandardStyles();

    // Connect signals...
    connect(borderWidget, SIGNAL(toggled(bool)), SLOT(borderToggled()));
    connect(menubarRoundTopOnlyWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(lighterPopupMenuBgndWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(roundWidget, SIGNAL(toggled(bool)), SLOT(roundToggled()));
    connect(toolbarBordersWidget, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(sliderThumbsCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(lvExpanderCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(lvHeaderColorCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(lvDarkWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(drawStatusBarFramesWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(lvHeaderAppearanceCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(handlesCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(appearanceCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(menuitemProfileCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(progressbarProfileCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(vArrowWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(emboldenWidget, SIGNAL(toggled(bool)), SLOT(emboldenToggled()));
    connect(crLabelHighlightWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(defaultIndicatorCombo, SIGNAL(activated(int)), SLOT(dbiChanged()));
    connect(borderSplitterWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(highlightTabWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(menubarAppearanceCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(toolbarAppearanceCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(toolbarSeparatorsCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(highlightEditsWidget, SIGNAL(toggled(bool)), SLOT(highlightEditsToggled()));
    connect(customSlidersColorButton, SIGNAL(changed( const QColor &)), SLOT(updateChanged()));
    connect(customMenubarsColorButton, SIGNAL(changed( const QColor &)), SLOT(updateChanged()));
    connect(customLvHeaderColorButton, SIGNAL(changed( const QColor &)), SLOT(updateChanged()));
    connect(shadeSlidersCombo, SIGNAL(activated(int)), SLOT(shadeSlidersChanged()));
    connect(shadeMenubarsCombo, SIGNAL(activated(int)), SLOT(shadeMenubarsChanged()));
    connect(lvHeaderColorCombo, SIGNAL(activated(int)), SLOT(lvHeaderColorChanged()));
    connect(focusCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(versionWidget, SIGNAL(selectionChanged()), SLOT(setVersion()));
    versionWidget->setSelected(currentStyle, true);
}

QtCurveConfig::~QtCurveConfig()
{
}

void QtCurveConfig::save()
{
    QSettings s;

    s.writeEntry("/qtcurvestyle/Settings/border", borderWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/round", roundWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/toolbarBorders", toStr((ETBarBorder)toolbarBordersWidget->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/appearance", toStr((EAppearance)appearanceCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/menuitemProfile", toStr((EProfile)menuitemProfileCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/progressbarProfile", toStr((EProfile)progressbarProfileCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/vArrow", vArrowWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/borderSplitter", borderSplitterWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/lighterPopupMenuBgnd", lighterPopupMenuBgndWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/embolden", emboldenWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/crLabelHighlight", crLabelHighlightWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/defBtnIndicator",
                 toStr((EDefBtnIndicator)defaultIndicatorCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/focus",
                 toStr((EFocus)focusCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/sliderThumbs", toStr((EGroove)sliderThumbsCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/handles", toStr((EHandle)handlesCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/lvExpander", toStr((ELvExpander)lvExpanderCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/lvLines", toStr((ELvLines)lvLinesCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/lvDark", lvDarkWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/drawStatusBarFrames", drawStatusBarFramesWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/lvHeaderAppearance", toStr((EAppearance)lvHeaderAppearanceCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/lvHeaderColor", toStr((EHeaderColor)lvHeaderColorCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/highlightTab", highlightTabWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/shadeSliders", toStr((EShade)shadeSlidersCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/shadeMenubars", toStr((EShade)shadeMenubarsCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/menubarRoundTopOnly", menubarRoundTopOnlyWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/menubarAppearance",
                 toStr((EAppearance)menubarAppearanceCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/toolbarAppearance",
                 toStr((EAppearance)toolbarAppearanceCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/toolbarSeparators",
                 toStr((ESeparator)toolbarSeparatorsCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/highlightEdits", highlightEditsWidget->isChecked());

    QColor col(customSlidersColorButton->color());
    char   colorStr[64];

    sprintf(colorStr, "#%02X%02X%02X", col.red(), col.green(), col.blue());
    s.writeEntry("/qtcurvestyle/Settings/customSlidersColor", colorStr);
    col=customMenubarsColorButton->color();
    sprintf(colorStr, "#%02X%02X%02X", col.red(), col.green(), col.blue());
    s.writeEntry("/qtcurvestyle/Settings/customMenubarsColor", colorStr);
    col=customLvHeaderColorButton->color();
    sprintf(colorStr, "#%02X%02X%02X", col.red(), col.green(), col.blue());
    s.writeEntry("/qtcurvestyle/Settings/customLvHeaderColor", colorStr);
}

void QtCurveConfig::defaults()
{
    versionWidget->setSelected(defaultStyle, true);
}

void QtCurveConfig::borderToggled()
{
    if(roundWidget->isChecked() && !borderWidget->isChecked())
    {
        roundWidget->setChecked(false);
        highlightEditsWidget->setChecked(false);
    }
    updateChanged();
}

void QtCurveConfig::highlightEditsToggled()
{
    if(highlightEditsWidget->isChecked() && !borderWidget->isChecked())
        borderWidget->setChecked(true);
    updateChanged();
}

void QtCurveConfig::roundToggled()
{
    if(roundWidget->isChecked() && !borderWidget->isChecked())
        borderWidget->setChecked(true);
    updateChanged();
}

void QtCurveConfig::emboldenToggled()
{
    if(!emboldenWidget->isChecked() && IND_NONE==defaultIndicatorCombo->currentItem())
        defaultIndicatorCombo->setCurrentItem(IND_CORNER);
    updateChanged();
}

void QtCurveConfig::dbiChanged()
{
    if(IND_NONE==defaultIndicatorCombo->currentItem() && !emboldenWidget->isChecked())
        emboldenWidget->setChecked(true);
    updateChanged();
}

void QtCurveConfig::lvChanged()
{
    lvDarkWidget->setEnabled(LV_LINES_NONE!=lvLinesCombo->currentItem());
    updateChanged();
}

void QtCurveConfig::shadeSlidersChanged()
{
    customSlidersColorButton->setEnabled(SHADE_CUSTOM==shadeSlidersCombo->currentItem());
    updateChanged();
}

void QtCurveConfig::shadeMenubarsChanged()
{
    customMenubarsColorButton->setEnabled(SHADE_CUSTOM==shadeMenubarsCombo->currentItem());
    updateChanged();
}

void QtCurveConfig::lvHeaderColorChanged()
{
    customLvHeaderColorButton->setEnabled(HEADER_COLOR_CUSTOM==lvHeaderColorCombo->currentItem());
    updateChanged();
}

void QtCurveConfig::updateChanged()
{
    if (settingsChanged())
        emit changed(true);
}

#define QTC_EXTENSION   ".qtcurve"
#define QTC_DESCR_GROUP "Description"

void QtCurveConfig::saveStyle()
{
    Options opts;

    setOptions(opts);
    saveStyle(KInputDialog::getText(i18n("Style Name"), i18n("Please enter a name for this style:"),
                                    0==versionWidget->selectedItem() || currentStyle==versionWidget->selectedItem()
                                        ? i18n("New Style")
                                        : versionWidget->selectedItem()->text(0), NULL, this), opts);
}

void QtCurveConfig::deleteStyle()
{
    ListViewItem *item=(ListViewItem *)(versionWidget->selectedItem());

    if(item && defaultStyle!=item &&
       !item->opts.file.isEmpty() &&
       KMessageBox::Yes==KMessageBox::warningYesNo(this, i18n("Delete the selected style?")))
        if(0==::unlink(item->opts.file.local8Bit().data()))
        {
            delete item;
            versionWidget->setSelected(currentStyle, true);
        }
        else
            KMessageBox::error(this, i18n("Could not delete style"));
}

void QtCurveConfig::importStyle()
{
    QString file(KFileDialog::getOpenFileName(QString::null, "*"QTC_EXTENSION, this));

    if(!file.isEmpty())
    {
        Options opts;

        if(loadOptions(file, opts) && !opts.name.isEmpty())
            saveStyle(opts.name, opts);
        else
            KMessageBox::error(this, i18n("This is not a QtCurve file!"));
    }
}

void QtCurveConfig::exportStyle()
{
    ListViewItem *item=(ListViewItem *)(versionWidget->selectedItem());

    if(!item)
        return;

    QString file(KFileDialog::getSaveFileName(0==versionWidget->selectedItem() ||
                                              currentStyle==versionWidget->selectedItem()
                                                  ? QString::null
                                                  : versionWidget->selectedItem()->text(0),
                                              "*"QTC_EXTENSION, this));

    if(!file.isEmpty())
    {
        QString title=item->text(0);

        if(!title.isEmpty())
        {
            KConfig cfg(file, false, false);

            if(cfg.isReadOnly())
                KMessageBox::error(this, i18n("Could not write to file:\n%1").arg(file));
            else
            {
                Options opts;

                setOptions(opts);
                opts.name=title;
                saveStyle(cfg, opts);
            }
        }
    }
}

void QtCurveConfig::setVersion()
{
    setVersion(versionWidget->selectedItem());
}

void QtCurveConfig::setVersion(QListViewItem *item)
{
    ListViewItem *i=(ListViewItem *)item;

    if(!i)
        i=currentStyle;

    setWidgetOptions(i->opts);
    lvDarkWidget->setEnabled(LV_LINES_NONE!=lvLinesCombo->currentItem());
    delBtn->setEnabled(item && !i->opts.readOnly);
    updateChanged();
}

void QtCurveConfig::saveStyle(const QString &title, Options &opts)
{
    if(!title.isEmpty())
    {
        if(currentStyle->opts.name==title)
        {
            KMessageBox::error(this,
                               i18n("Sorry, you can't create a style named \"%1\", this is reseved")
                               .arg(title));
            return;
        }

        QListViewItem *item=versionWidget->firstChild();

        while(item)
        {
            if (item->text(0)==title)
                if(((ListViewItem *)item)->opts.readOnly)
                {
                    KMessageBox::error(this,
                                       i18n("Sorry, a style named \"%1\" already exists, and is read-only")
                                       .arg(title));
                    return;
                }
                else
                    if(KMessageBox::No==KMessageBox::warningYesNo(this, i18n("Overwrite the existing style?")))
                        return;
                    else
                        break;
            item=item->nextSibling();
        }

        QString resource("qtcurve/"+title+QTC_EXTENSION);
        KConfig cfg(resource, false, false, "data");

        opts.name=title;
        if(saveStyle(cfg, opts))
        {
            QString file(::locate("data", resource));

            if(!file.isEmpty())
            {
                Options opts;

                if(loadOptions(file, opts) && !opts.name.isEmpty())
                    if(item)
                        ((ListViewItem *)item)->opts=opts;
                    else
                        new ListViewItem(opts, versionWidget);
            }
        }
    }
}

bool QtCurveConfig::saveStyle(KConfig &cfg, const Options &opts)
{
    if(cfg.isReadOnly())
        KMessageBox::error(this, i18n("Could not save style"));
    else
    {
        cfg.setGroup(QTC_GROUP);
        cfg.writeEntry("border", opts.border);
        cfg.writeEntry("round", opts.round);
        cfg.writeEntry("toolbarBorders", toStr(opts.toolbarBorders));
        cfg.writeEntry("appearance", toStr(opts.appearance));
        cfg.writeEntry("focus", toStr(opts.focus));
        cfg.writeEntry("vArrow", opts.vArrow);
        cfg.writeEntry("borderSplitter", opts.borderSplitter);
        cfg.writeEntry("lighterPopupMenuBgnd", opts.lighterPopupMenuBgnd);
        cfg.writeEntry("embolden", opts.embolden);
        cfg.writeEntry("crLabelHighlight", opts.crLabelHighlight);
        cfg.writeEntry("defBtnIndicator", toStr(opts.defBtnIndicator));
        cfg.writeEntry("sliderThumbs", toStr(opts.sliderThumbs));
        cfg.writeEntry("handles", toStr(opts.handles));
        cfg.writeEntry("lvExpander", toStr(opts.lvExpander));
        cfg.writeEntry("lvLines", toStr(opts.lvLines));
        cfg.writeEntry("lvDark", opts.lvDark);
        cfg.writeEntry("drawStatusBarFrames", opts.drawStatusBarFrames);
        cfg.writeEntry("lvHeaderAppearance", toStr(opts.lvHeaderAppearance));
        cfg.writeEntry("lvHeaderColor", toStr(opts.lvHeaderColor));
        cfg.writeEntry("menuitemProfile", toStr(opts.menuitemProfile));
        cfg.writeEntry("progressbarProfile", toStr(opts.progressbarProfile));
        cfg.writeEntry("highlightTab", opts.highlightTab);
        cfg.writeEntry("shadeSliders", toStr(opts.shadeSliders));
        cfg.writeEntry("shadeMenubars", toStr(opts.shadeMenubars));
        cfg.writeEntry("menubarAppearance", toStr(opts.menubarAppearance));
        cfg.writeEntry("menubarRoundTopOnly", opts.menubarRoundTopOnly);
        cfg.writeEntry("toolbarAppearance", toStr(opts.toolbarAppearance));
        cfg.writeEntry("toolbarSeparators", toStr(opts.toolbarSeparators));
        cfg.writeEntry("highlightEdits", opts.highlightEdits);

        char colorStr[64];

        sprintf(colorStr, "#%02X%02X%02X", opts.customSlidersColor.red(),
                                           opts.customSlidersColor.green(),
                                           opts.customSlidersColor.blue());
        cfg.writeEntry("customSlidersColor", colorStr);
        sprintf(colorStr, "#%02X%02X%02X", opts.customMenubarsColor.red(),
                                           opts.customMenubarsColor.green(),
                                           opts.customMenubarsColor.blue());
        cfg.writeEntry("customMenubarsColor", colorStr);
        sprintf(colorStr, "#%02X%02X%02X", opts.customLvHeaderColor.red(),
                                           opts.customLvHeaderColor.green(),
                                           opts.customLvHeaderColor.blue());
        cfg.writeEntry("customLvHeaderColor", colorStr);
        cfg.setGroup(QTC_DESCR_GROUP);
        cfg.writeEntry("name", opts.name);
        cfg.writeEntry("default", false);

        cfg.sync();
        return true;
    }

    return false;
}

void QtCurveConfig::loadStandardStyles()
{
    QStringList           files=KGlobal::dirs()->findAllResources("data",
                                                                  "qtcurve/*"QTC_EXTENSION, false, true);
    QStringList::Iterator it(files.begin()),
                          end(files.end());
    Options               opts;

    opts.loadSettings();
    opts.readOnly=true;
    opts.name=i18n("Current Scheme");
    currentStyle=new ListViewItem(opts, versionWidget, true);

    for(; it!=end; ++it)
    {
        Options stdOpts;

        if(loadOptions(*it, stdOpts) && !stdOpts.name.isEmpty())
        {
            QListViewItem *i=versionWidget->firstChild();

            while(i)
            {
                if (i->text(0)==stdOpts.name)
                    break;
                else
                    i=i->nextSibling();
            }

            if(!i)
            {
                ListViewItem *item=new ListViewItem(stdOpts, versionWidget);
                if(stdOpts.def)
                    defaultStyle=item;
            }
        }
    }

    if(!defaultStyle)
        defaultStyle=currentStyle;
}

void QtCurveConfig::setOptions(Options &opts)
{
    opts.border=borderWidget->isChecked();
    opts.round=roundWidget->isChecked();
    opts.toolbarBorders=(ETBarBorder)toolbarBordersWidget->currentItem();
    opts.appearance=(EAppearance)appearanceCombo->currentItem();
    opts.menuitemProfile=(EProfile)menuitemProfileCombo->currentItem();
    opts.progressbarProfile=(EProfile)progressbarProfileCombo->currentItem();
    opts.vArrow=vArrowWidget->isChecked();
    opts.borderSplitter=borderSplitterWidget->isChecked();
    opts.lighterPopupMenuBgnd=lighterPopupMenuBgndWidget->isChecked();
    opts.embolden=emboldenWidget->isChecked();
    opts.focus=(EFocus)focusCombo->currentItem();
    opts.crLabelHighlight=crLabelHighlightWidget->isChecked();
    opts.defBtnIndicator=(EDefBtnIndicator)defaultIndicatorCombo->currentItem();
    opts.sliderThumbs=(EGroove)sliderThumbsCombo->currentItem();
    opts.handles=(EHandle)handlesCombo->currentItem();
    opts.lvExpander=(ELvExpander)lvExpanderCombo->currentItem();
    opts.lvLines=(ELvLines)lvLinesCombo->currentItem();
    opts.lvDark=lvDarkWidget->isChecked();
    opts.drawStatusBarFrames=drawStatusBarFramesWidget->isChecked();
    opts.lvHeaderAppearance=(EAppearance)lvHeaderAppearanceCombo->currentItem();
    opts.lvHeaderColor=(EHeaderColor)lvHeaderColorCombo->currentItem();
    opts.highlightTab=highlightTabWidget->isChecked();
    opts.shadeSliders=(EShade)shadeSlidersCombo->currentItem();
    opts.shadeMenubars=(EShade)shadeMenubarsCombo->currentItem();
    opts.menubarAppearance=(EAppearance)menubarAppearanceCombo->currentItem();
    opts.menubarRoundTopOnly=menubarRoundTopOnlyWidget->isChecked();
    opts.toolbarAppearance=(EAppearance)toolbarAppearanceCombo->currentItem();
    opts.toolbarSeparators=(ESeparator)toolbarSeparatorsCombo->currentItem();
    opts.highlightEdits=highlightEditsWidget->isChecked();
    opts.customSlidersColor=customSlidersColorButton->color();
    opts.customMenubarsColor=customMenubarsColorButton->color();
    opts.customLvHeaderColor=customLvHeaderColorButton->color();
}

void QtCurveConfig::setWidgetOptions(const Options &opts)
{
    focusCombo->setCurrentItem(opts.focus);
    borderWidget->setChecked(opts.border);
    lighterPopupMenuBgndWidget->setChecked(opts.lighterPopupMenuBgnd);
    roundWidget->setChecked(opts.round);
    toolbarBordersWidget->setCurrentItem(opts.toolbarBorders);
    sliderThumbsCombo->setCurrentItem(opts.sliderThumbs);
    lvExpanderCombo->setCurrentItem(opts.lvExpander);
    lvLinesCombo->setCurrentItem(opts.lvLines);
    lvDarkWidget->setChecked(opts.lvDark);
    drawStatusBarFramesWidget->setChecked(opts.drawStatusBarFrames);
    lvHeaderAppearanceCombo->setCurrentItem(opts.lvHeaderAppearance);
    lvHeaderColorCombo->setCurrentItem(opts.lvHeaderColor);
    handlesCombo->setCurrentItem(opts.handles);
    appearanceCombo->setCurrentItem(opts.appearance);
    menuitemProfileCombo->setCurrentItem(opts.menuitemProfile);
    progressbarProfileCombo->setCurrentItem(opts.progressbarProfile);
    vArrowWidget->setChecked(opts.vArrow);
    emboldenWidget->setChecked(opts.embolden);
    crLabelHighlightWidget->setChecked(opts.crLabelHighlight);
    defaultIndicatorCombo->setCurrentItem(opts.defBtnIndicator);
    borderSplitterWidget->setChecked(opts.borderSplitter);
    highlightTabWidget->setChecked(opts.highlightTab);
    menubarRoundTopOnlyWidget->setChecked(opts.menubarRoundTopOnly);
    menubarAppearanceCombo->setCurrentItem(opts.menubarAppearance);
    toolbarAppearanceCombo->setCurrentItem(opts.toolbarAppearance);
    toolbarSeparatorsCombo->setCurrentItem(opts.toolbarSeparators);
    highlightEditsWidget->setChecked(opts.highlightEdits);
    shadeSlidersCombo->setCurrentItem(opts.shadeSliders);
    shadeMenubarsCombo->setCurrentItem(opts.shadeMenubars);

    customSlidersColorButton->setColor(opts.customSlidersColor);
    customMenubarsColorButton->setColor(opts.customMenubarsColor);
    customLvHeaderColorButton->setColor(opts.customLvHeaderColor);

    customSlidersColorButton->setEnabled(SHADE_CUSTOM==opts.shadeSliders);
    customMenubarsColorButton->setEnabled(SHADE_CUSTOM==opts.shadeMenubars);
    customLvHeaderColorButton->setEnabled(HEADER_COLOR_CUSTOM==opts.lvHeaderColor);
}

bool QtCurveConfig::loadOptions(const QString &file, Options &opts)
{
    KConfig cfg(file, false, false);

    if(cfg.hasGroup(QTC_GROUP) && cfg.hasGroup(QTC_DESCR_GROUP))
    {
        cfg.setGroup(QTC_GROUP);
        opts.border=cfg.readBoolEntry("border", currentStyle->opts.border);
        opts.lighterPopupMenuBgnd=cfg.readBoolEntry("lighterPopupMenuBgnd",
                                              currentStyle->opts.lighterPopupMenuBgnd);
        opts.round=cfg.readBoolEntry("round", currentStyle->opts.round);
        opts.highlightEdits=cfg.readBoolEntry("highlightEdits", currentStyle->opts.highlightEdits);
        opts.toolbarBorders=qtc_to_tbar_border(cfg.readEntry("toolbarBorders",
                                              toStr(currentStyle->opts.toolbarBorders)).latin1());

        opts.sliderThumbs=qtc_to_groove(cfg.readEntry("sliderThumbs",
                                           toStr(currentStyle->opts.sliderThumbs)).latin1());
        opts.lvExpander=qtc_to_lv_expander(cfg.readEntry("lvExpander",
                                           toStr(currentStyle->opts.lvExpander)).latin1());
        opts.lvLines=qtc_to_lv_lines(cfg.readEntry("lvLines",
                                           toStr(currentStyle->opts.lvLines)).latin1());
        opts.lvDark=cfg.readBoolEntry("lvDark", currentStyle->opts.lvDark);
        opts.drawStatusBarFrames=cfg.readBoolEntry("drawStatusBarFrames", currentStyle->opts.drawStatusBarFrames);
        opts.lvHeaderAppearance=qtc_to_appearance(cfg.readEntry("lvHeaderAppearance",
                                           toStr(currentStyle->opts.lvHeaderAppearance)).latin1());
        opts.lvHeaderColor=qtc_to_header_color(cfg.readEntry("lvHeaderColor",
                                           toStr(currentStyle->opts.lvHeaderColor)).latin1());
        opts.handles=qtc_to_handle(cfg.readEntry("handles",
                                           toStr(currentStyle->opts.handles)).latin1());
        opts.appearance=qtc_to_appearance(cfg.readEntry("appearance",
                                           toStr(currentStyle->opts.appearance)).latin1());

        opts.focus=qtc_to_focus(cfg.readEntry("focus", toStr(currentStyle->opts.focus)).latin1());
        opts.menubarRoundTopOnly=cfg.readBoolEntry("menubarRoundTopOnly", currentStyle->opts.menubarRoundTopOnly);
        EAppearance app=qtc_to_appearance(cfg.readEntry("menubarAppearance",
                                          toStr(currentStyle->opts.menubarAppearance)).latin1());
        if(APPEARANCE_LIGHT_GRADIENT==app)
          app=APPEARANCE_GRADIENT;
        opts.menubarAppearance=app;

        app=qtc_to_appearance(cfg.readEntry("toolbarAppearance",
                                          toStr(currentStyle->opts.toolbarAppearance)).latin1());
        if(APPEARANCE_LIGHT_GRADIENT==app)
          app=APPEARANCE_GRADIENT;
        opts.toolbarAppearance=app;

        QString pm=cfg.readEntry("pm", DEF_PROFILE_STR).latin1();
        opts.menuitemProfile=qtc_to_profile(
                              cfg.readEntry("menuitemProfile", pm).latin1());
        opts.progressbarProfile=qtc_to_profile(
                                 cfg.readEntry("progressbarProfile", pm).latin1());

        opts.vArrow=cfg.readBoolEntry("vArrow", currentStyle->opts.vArrow);
        opts.embolden=cfg.readBoolEntry("embolden", currentStyle->opts.embolden);
        opts.crLabelHighlight=cfg.readBoolEntry("crLabelHighlight", currentStyle->opts.crLabelHighlight);

        opts.defBtnIndicator=qtc_to_ind(cfg.readEntry("defBtnIndicator",
                                        toStr(currentStyle->opts.defBtnIndicator)).latin1());

        opts.borderSplitter=cfg.readBoolEntry("borderSplitter", currentStyle->opts.borderSplitter);
        opts.highlightTab=cfg.readBoolEntry("highlightTab", currentStyle->opts.highlightTab);

        opts.toolbarSeparators=qtc_to_separator(cfg.readEntry("toolbarSeparators",
                                                toStr(currentStyle->opts.toolbarSeparators)).latin1());

        opts.shadeSliders=qtc_to_shade(cfg.readEntry("shadeSliders",
                                           toStr(currentStyle->opts.shadeSliders)).latin1(), false);

        QColor col(currentStyle->opts.customSlidersColor);

        opts.customSlidersColor=cfg.readColorEntry("customSlidersColor", &col);
        opts.shadeMenubars=qtc_to_shade(cfg.readEntry("shadeMenubars",
                                           toStr(currentStyle->opts.shadeMenubars)).latin1(), true);
        col=currentStyle->opts.customMenubarsColor;
        opts.customMenubarsColor=cfg.readColorEntry("customMenubarsColor", &col);
        col=currentStyle->opts.customLvHeaderColor;
        opts.customLvHeaderColor=cfg.readColorEntry("customLvHeaderColor", &col);

        cfg.setGroup(QTC_DESCR_GROUP);

        opts.name=cfg.readEntry("name");
        opts.def=cfg.readBoolEntry("default", false);

        opts.readOnly=cfg.isReadOnly();
        opts.file=file;
        return true;
    }
    return false;
}

bool QtCurveConfig::settingsChanged()
{
    return borderWidget->isChecked()!=currentStyle->opts.border ||
         roundWidget->isChecked()!=currentStyle->opts.round ||
         toolbarBordersWidget->currentItem()!=currentStyle->opts.toolbarBorders ||
         appearanceCombo->currentItem()!=(int)currentStyle->opts.appearance ||
         menuitemProfileCombo->currentItem()!=(int)currentStyle->opts.menuitemProfile ||
         progressbarProfileCombo->currentItem()!=(int)currentStyle->opts.progressbarProfile ||
         vArrowWidget->isChecked()!=currentStyle->opts.vArrow ||
         borderSplitterWidget->isChecked()!=currentStyle->opts.borderSplitter ||
         lighterPopupMenuBgndWidget->isChecked()!=currentStyle->opts.lighterPopupMenuBgnd ||
         emboldenWidget->isChecked()!=currentStyle->opts.embolden ||
         crLabelHighlightWidget->isChecked()!=currentStyle->opts.crLabelHighlight ||
         defaultIndicatorCombo->currentItem()!=(int)currentStyle->opts.defBtnIndicator ||
         sliderThumbsCombo->currentItem()!=(int)currentStyle->opts.sliderThumbs ||
         handlesCombo->currentItem()!=(int)currentStyle->opts.handles ||
         lvExpanderCombo->currentItem()!=(int)currentStyle->opts.lvExpander ||
         lvLinesCombo->currentItem()!=(int)currentStyle->opts.lvLines ||
         focusCombo->currentItem()!=(int)currentStyle->opts.focus ||
         lvDarkWidget->isChecked()!=currentStyle->opts.lvDark ||
         drawStatusBarFramesWidget->isChecked()!=currentStyle->opts.drawStatusBarFrames ||
         lvHeaderAppearanceCombo->currentItem()!=(int)currentStyle->opts.lvHeaderAppearance ||
         highlightTabWidget->isChecked()!=currentStyle->opts.highlightTab ||
         shadeSlidersCombo->currentItem()!=(int)currentStyle->opts.shadeSliders ||
         shadeMenubarsCombo->currentItem()!=(int)currentStyle->opts.shadeMenubars ||
         menubarRoundTopOnlyWidget->isChecked()!=currentStyle->opts.menubarRoundTopOnly ||
         menubarAppearanceCombo->currentItem()!=currentStyle->opts.menubarAppearance ||
         toolbarAppearanceCombo->currentItem()!=currentStyle->opts.toolbarAppearance ||
         toolbarSeparatorsCombo->currentItem()!=currentStyle->opts.toolbarSeparators ||
         highlightEditsWidget->isChecked()!=currentStyle->opts.highlightEdits ||
         lvHeaderColorCombo->currentItem()!=(int)currentStyle->opts.lvHeaderColor ||
         (HEADER_COLOR_CUSTOM==currentStyle->opts.lvHeaderColor &&
               customLvHeaderColorButton->color()!=currentStyle->opts.customLvHeaderColor) ||
         (SHADE_CUSTOM==currentStyle->opts.shadeSliders &&
               customSlidersColorButton->color()!=currentStyle->opts.customSlidersColor) ||
         (SHADE_CUSTOM==currentStyle->opts.shadeMenubars &&
               customMenubarsColorButton->color()!=currentStyle->opts.customMenubarsColor);
}

#include "qtcurveconfig.moc"
